@@ -3,6 +3,7 @@ Execution abstraction.
 
 """
 from multiprocessing import Pool
+from signal import SIGINT, SIGTERM, signal
 
 
 class SimpleRunner:
@@ -35,14 +36,35 @@ class ProcessRunner:
         self.target = target
         self.args = args
         self.kwargs = kwargs
+        self.pool = None
+
+        self.init_signal_handlers()
 
     def run(self):
-        pool = Pool(processes=self.processes)
-        for _ in range(self.processes):
-            pool.apply_async(_start, (self.target,) + self.args, self.kwargs)
+        self.pool = self.process_pool()
 
-        pool.close()
-        try:
-            pool.join()
-        except KeyboardInterrupt:
-            pool.join()
+        for _ in range(self.processes):
+            self.pool.apply_async(_start, (self.target,) + self.args, self.kwargs)
+
+        self.close()
+
+    def init_signal_handlers(self):
+        for signum in (SIGINT, SIGTERM):
+            signal(signum, self.on_terminate)
+
+    def process_pool(self):
+        return Pool(processes=self.processes)
+
+    def close(self, terminate=False):
+        if self.pool is not None:
+            if terminate:
+                self.pool.terminate()
+            else:
+                self.pool.close()
+
+            self.pool.join()
+
+        exit(0)
+
+    def on_terminate(self, signum, frame):
+        self.close(terminate=True)
